@@ -2,31 +2,35 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
-use crate::photo::model::{Picture, PictureId, ScannedFile};
+use crate::FlatpakPathBuf;
+use crate::ScannedFile;
+use crate::path_encoding;
+use crate::people::model::{DetectedFace, FaceDetectionCandidate, FaceId, Rect};
+use crate::photo::model::{Picture, PictureId};
 
 use super::Metadata;
 use super::metadata;
 use super::model::MotionPhotoVideo;
 use super::motion_photo;
-use crate::path_encoding;
 use anyhow::{Result, bail};
 use rusqlite;
 use rusqlite::Row;
 use rusqlite::params;
 use std::path::{Path, PathBuf};
 use std::sync::{Arc, Mutex};
+use tracing::error;
 
 /// Repository of picture metadata.
 /// Repository is backed by a Sqlite database.
 #[derive(Debug, Clone)]
 pub struct Repository {
-    /// Base path to picture library on file system
-    library_base_path: PathBuf,
+    /// Base path to library
+    library_base_dir: FlatpakPathBuf,
 
-    /// Base path cache directory for photo thumbnails and motion photo videos
+    /// Base path cache directory for motion photo videos
     cache_dir_base_path: PathBuf,
 
-    /// Base path for data directory
+    /// Base path cache directory for motion photo videos
     data_dir_base_path: PathBuf,
 
     /// Connection to backing Sqlite database.
@@ -36,23 +40,19 @@ pub struct Repository {
 impl Repository {
     /// Builds a Repository and creates operational tables.
     pub fn open(
-        library_base_path: &Path,
+        library_base_dir: &FlatpakPathBuf,
         cache_dir_base_path: &Path,
         data_dir_base_path: &Path,
         con: Arc<Mutex<rusqlite::Connection>>,
     ) -> Result<Repository> {
-        if !library_base_path.is_dir() {
-            bail!("{:?} is not a directory", library_base_path);
+        if !library_base_dir.sandbox_path.is_dir() {
+            bail!("{:?} is not a directory", library_base_dir);
         }
 
-        let library_base_path = PathBuf::from(library_base_path);
-        let cache_dir_base_path = PathBuf::from(cache_dir_base_path);
-        let data_dir_base_path = PathBuf::from(data_dir_base_path);
-
         let repo = Repository {
-            library_base_path,
-            cache_dir_base_path,
-            data_dir_base_path,
+            library_base_dir: library_base_dir.clone(),
+            cache_dir_base_path: cache_dir_base_path.into(),
+            data_dir_base_path: data_dir_base_path.into(),
             con,
         };
 
@@ -117,32 +117,6 @@ impl Repository {
         Ok(())
     }
 
-    pub fn add_thumbnail(&mut self, picture_id: &PictureId, thumbnail_path: &Path) -> Result<()> {
-        let mut con = self.con.lock().unwrap();
-        let tx = con.transaction()?;
-
-        {
-            let mut stmt = tx.prepare_cached(
-                "UPDATE pictures
-                SET
-                    thumbnail_path = ?2,
-                    is_broken = FALSE
-                WHERE picture_id = ?1",
-            )?;
-
-            // convert to relative path before saving to database
-            let thumbnail_path = thumbnail_path.strip_prefix(&self.cache_dir_base_path).ok();
-
-            stmt.execute(params![
-                picture_id.id(),
-                thumbnail_path.as_ref().map(|p| p.to_str()),
-            ])?;
-        }
-
-        tx.commit()?;
-        Ok(())
-    }
-
     pub fn mark_broken(&mut self, picture_id: &PictureId) -> Result<()> {
         let mut con = self.con.lock().unwrap();
         let tx = con.transaction()?;
@@ -185,28 +159,34 @@ impl Repository {
                 ",
             )?;
 
-            for pic in pics {
-                // convert to relative path before saving to database
-                let picture_path = pic.path.strip_prefix(&self.library_base_path)?;
-                let picture_path_b64 = path_encoding::to_base64(picture_path);
+            for scanned_file in pics {
+                if let ScannedFile::Photo(info) = scanned_file {
+                    // convert to relative path before saving to database
+                    let picture_path = info
+                        .path
+                        .strip_prefix(&self.library_base_dir.sandbox_path)?;
+                    let picture_path_b64 = path_encoding::to_base64(picture_path);
 
-                // Path without suffix so sibling pictures and videos can be related
-                let link_path = picture_path
-                    .file_stem()
-                    .and_then(|x| x.to_str())
-                    .expect("Must exist");
+                    // Path without suffix so sibling pictures and videos can be related
+                    let link_path = picture_path
+                        .file_stem()
+                        .and_then(|x| x.to_str())
+                        .expect("Must exist");
 
-                let link_path = picture_path.with_file_name(link_path);
-                let link_path_b64 = path_encoding::to_base64(&link_path);
+                    let link_path = picture_path.with_file_name(link_path);
+                    let link_path_b64 = path_encoding::to_base64(&link_path);
 
-                pic_insert_stmt.execute(params![
-                    pic.fs_created_at,
-                    pic.fs_modified_at,
-                    picture_path_b64,
-                    picture_path.to_string_lossy(),
-                    link_path_b64,
-                    link_path.to_string_lossy(),
-                ])?;
+                    pic_insert_stmt.execute(params![
+                        info.fs_created_at,
+                        info.fs_modified_at,
+                        picture_path_b64,
+                        picture_path.to_string_lossy(),
+                        link_path_b64,
+                        link_path.to_string_lossy(),
+                    ])?;
+                } else {
+                    error!("Expected a photo, but got: {:?}", scanned_file);
+                }
             }
         }
 
@@ -221,7 +201,6 @@ impl Repository {
             "SELECT
                     pictures.picture_id,
                     pictures.picture_path_b64,
-                    pictures.thumbnail_path,
                     COALESCE(
                         pictures.exif_created_ts,
                         pictures.exif_modified_ts,
@@ -252,7 +231,6 @@ impl Repository {
             "SELECT
                     pictures.picture_id,
                     pictures.picture_path_b64,
-                    pictures.thumbnail_path,
                     COALESCE(
                         pictures.exif_created_ts,
                         pictures.exif_modified_ts,
@@ -282,7 +260,6 @@ impl Repository {
             "SELECT
                     pictures.picture_id,
                     pictures.picture_path_b64,
-                    pictures.thumbnail_path,
                     COALESCE(
                         pictures.exif_created_ts,
                         pictures.exif_modified_ts,
@@ -398,23 +375,19 @@ impl Repository {
     fn to_picture(&self, row: &Row<'_>) -> rusqlite::Result<Picture> {
         let picture_id = row.get("picture_id").map(PictureId::new)?;
 
-        let picture_path: String = row.get("picture_path_b64")?;
-        let picture_path =
-            path_encoding::from_base64(&picture_path).map_err(|_| rusqlite::Error::InvalidQuery)?;
-        let picture_path = self.library_base_path.join(picture_path);
+        let relative_path: String = row.get("picture_path_b64")?;
+        let relative_path = path_encoding::from_base64(&relative_path)
+            .map_err(|_| rusqlite::Error::InvalidQuery)?;
 
-        let thumbnail_path = row
-            .get("thumbnail_path")
-            .map(|p: String| self.cache_dir_base_path.join(p))
-            .ok();
+        let sandbox_path = self.library_base_dir.sandbox_path.join(&relative_path);
+        let host_path = self.library_base_dir.host_path.join(&relative_path);
 
         let ordering_ts = row.get("ordering_ts").expect("must have ordering_ts");
         let is_selfie = row.get("is_selfie").ok();
 
         std::result::Result::Ok(Picture {
             picture_id,
-            path: picture_path,
-            thumbnail_path,
+            path: FlatpakPathBuf::build(host_path, sandbox_path),
             ordering_ts,
             is_selfie,
         })
@@ -425,6 +398,7 @@ impl Repository {
 
         row.get("path")
             .and_then(|p: String| match root_name.as_str() {
+                // FIXME what about thumbnail path?
                 "cache" => std::result::Result::Ok(self.cache_dir_base_path.join(p)),
                 "data" => std::result::Result::Ok(self.data_dir_base_path.join(p)),
                 _ => Err(rusqlite::Error::InvalidPath(p.into())),
@@ -440,10 +414,137 @@ impl Repository {
         Ok(())
     }
 
+    /// Find all people and associated face.
+    /// FIXME move to people repo
+    pub fn find_people_for_thumbnails(&self) -> Result<Vec<(FlatpakPathBuf, DetectedFace)>> {
+        let con = self.con.lock().unwrap();
+
+        // NOTE: this is non-standard SQL that might not work in DBs that aren't SQLite.
+        let mut stmt = con.prepare(
+            "SELECT
+                face_id,
+                detected_at,
+
+                is_source_original,
+                pictures.picture_path_b64 AS picture_path_b64,
+
+                bounds_path,
+                faces.thumbnail_path AS thumbnail_path,
+
+                bounds_x,
+                bounds_y,
+                bounds_width,
+                bounds_height,
+
+                right_eye_x,
+                right_eye_y,
+
+                left_eye_x,
+                left_eye_y,
+
+                nose_x,
+                nose_y,
+
+                right_mouth_corner_x,
+                right_mouth_corner_y,
+
+                left_mouth_corner_x,
+                left_mouth_corner_y,
+
+                confidence
+            FROM  pictures_faces AS faces
+            INNER JOIN pictures USING (picture_id)
+            WHERE faces.person_id IS NOT NULL
+            AND faces.is_thumbnail IS TRUE",
+        )?;
+
+        let result: Vec<(FlatpakPathBuf, DetectedFace)> = stmt
+            .query_map([], |row| {
+                Ok((self.to_library_path(row)?, self.to_detected_face(row)?))
+            })?
+            .flatten()
+            .collect();
+
+        Ok(result)
+    }
+
+    fn to_library_path(&self, row: &Row<'_>) -> rusqlite::Result<FlatpakPathBuf> {
+        let relative_path: String = row.get("picture_path_b64")?;
+        let relative_path = path_encoding::from_base64(&relative_path)
+            .map_err(|_| rusqlite::Error::InvalidQuery)?;
+
+        let sandbox_path = self.library_base_dir.sandbox_path.join(&relative_path);
+        let host_path = self.library_base_dir.host_path.join(&relative_path);
+
+        std::result::Result::Ok(FlatpakPathBuf {
+            host_path,
+            sandbox_path,
+        })
+    }
+
+    /// FIXME a copy-n-paste from people repo :-()
+    fn to_detected_face(&self, row: &Row<'_>) -> rusqlite::Result<DetectedFace> {
+        let face_id = row.get("face_id").map(FaceId::new)?;
+
+        let face_path = row
+            .get("bounds_path")
+            .map(|p: String| self.data_dir_base_path.join(p))?;
+
+        let thumbnail_path = row
+            .get("thumbnail_path")
+            .map(|p: String| self.data_dir_base_path.join(p))?;
+
+        let bounds = Rect {
+            x: row.get("bounds_x")?,
+            y: row.get("bounds_y")?,
+            width: row.get("bounds_width")?,
+            height: row.get("bounds_height")?,
+        };
+
+        let right_eye_x = row.get("right_eye_x")?;
+        let right_eye_y = row.get("right_eye_y")?;
+
+        let left_eye_x = row.get("left_eye_x")?;
+        let left_eye_y = row.get("left_eye_y")?;
+
+        let nose_x = row.get("nose_x")?;
+        let nose_y = row.get("nose_y")?;
+
+        let right_mouth_corner_x = row.get("right_mouth_corner_x")?;
+        let right_mouth_corner_y = row.get("right_mouth_corner_y")?;
+
+        let left_mouth_corner_x = row.get("left_mouth_corner_x")?;
+        let left_mouth_corner_y = row.get("left_mouth_corner_y")?;
+
+        let confidence = row.get("confidence")?;
+
+        let detected_at = row.get("detected_at")?;
+
+        let is_source_original: bool = row.get("is_source_original")?;
+
+        let face = DetectedFace {
+            face_id,
+            face_path,
+            small_thumbnail_path: thumbnail_path,
+            is_source_original,
+            bounds,
+            right_eye: (right_eye_x, right_eye_y),
+            left_eye: (left_eye_x, left_eye_y),
+            nose: (nose_x, nose_y),
+            right_mouth_corner: (right_mouth_corner_x, right_mouth_corner_y),
+            left_mouth_corner: (left_mouth_corner_x, left_mouth_corner_y),
+            confidence,
+            detected_at,
+        };
+
+        std::result::Result::Ok(face)
+    }
+
     /// Gets all pictures that haven't been scanned for faces.
     /// This method is not on the people repo because I don't what that repo
     /// to need a pic_base_dir.
-    pub fn find_need_face_scan(&self) -> Result<Vec<(PictureId, PathBuf)>> {
+    /// FIXME move to people repo
+    pub fn find_face_detection_candidates(&self) -> Result<Vec<FaceDetectionCandidate>> {
         let con = self.con.lock().unwrap();
         let mut stmt = con.prepare(
             "SELECT
@@ -464,14 +565,21 @@ impl Repository {
         )?;
 
         let result = stmt
-            .query_map([], |row| self.to_picture_id_path_tuple(row))?
+            .query_map([], |row| self.to_face_detection_candidate(row))?
             .flatten()
             .collect();
 
         Ok(result)
     }
 
-    pub fn get_picture_path(&self, picture_id: PictureId) -> Result<Option<PathBuf>> {
+    /// Gets all pictures that haven't been scanned for faces.
+    /// This method is not on the people repo because I don't what that repo
+    /// to need a pic_base_dir.
+    /// FIXME move to people repo
+    pub fn get_face_detection_candidate(
+        &self,
+        picture_id: &PictureId,
+    ) -> Result<Option<FaceDetectionCandidate>> {
         let con = self.con.lock().unwrap();
         let mut stmt = con.prepare(
             "SELECT
@@ -482,22 +590,37 @@ impl Repository {
         )?;
 
         let result = stmt
-            .query_map([picture_id.id()], |row| self.to_picture_id_path_tuple(row))?
+            .query_map([picture_id.id()], |row| {
+                self.to_face_detection_candidate(row)
+            })?
             .flatten()
-            .nth(0)
-            .map(|x| x.1);
+            .nth(0);
 
         Ok(result)
     }
 
-    fn to_picture_id_path_tuple(&self, row: &Row<'_>) -> rusqlite::Result<(PictureId, PathBuf)> {
+    /// FIXME move to people repo
+    fn to_face_detection_candidate(
+        &self,
+        row: &Row<'_>,
+    ) -> rusqlite::Result<FaceDetectionCandidate> {
         let picture_id = row.get("picture_id").map(PictureId::new)?;
 
-        let picture_path: String = row.get("picture_path_b64")?;
-        let picture_path =
-            path_encoding::from_base64(&picture_path).map_err(|_| rusqlite::Error::InvalidQuery)?;
-        let picture_path = self.library_base_path.join(picture_path);
+        let relative_path = row
+            .get("picture_path_b64")
+            .ok()
+            .and_then(|x: String| path_encoding::from_base64(&x).ok());
 
-        std::result::Result::Ok((picture_id, picture_path))
+        let host_path = relative_path
+            .as_ref()
+            .map(|x| self.library_base_dir.host_path.join(x));
+
+        let sandbox_path = relative_path.map(|x| self.library_base_dir.sandbox_path.join(x));
+
+        Ok(FaceDetectionCandidate {
+            picture_id,
+            host_path: host_path.expect("Must have host path"),
+            sandbox_path: sandbox_path.expect("Must have sandbox path"),
+        })
     }
 }
