@@ -2,14 +2,16 @@
 //
 // SPDX-License-Identifier: GPL-3.0-or-later
 
+use relm4::adw::prelude::*;
+use relm4::gtk;
+use relm4::prelude::*;
+
 use relm4::{
-    Component, ComponentController, ComponentParts, ComponentSender, Controller, SimpleComponent,
+    Component, ComponentController, Controller,
     WorkerController,
     actions::{RelmAction, RelmActionGroup},
     adw,
-    adw::prelude::{AdwApplicationWindowExt, NavigationPageExt},
     component::{AsyncComponent, AsyncComponentController},
-    gtk,
     gtk::{
         gio, glib,
         prelude::{ApplicationExt, ButtonExt, GtkWindowExt, OrientableExt, SettingsExt, WidgetExt},
@@ -28,12 +30,15 @@ use fotema_core::VisualId;
 use fotema_core::database;
 use fotema_core::path_encoding;
 use fotema_core::people;
+use fotema_core::thumbnailify::Thumbnailer;
+use fotema_core::FlatpakPathBuf;
 
 use h3o::CellIndex;
 
 use std::path::PathBuf;
 use std::str::FromStr;
 use std::sync::{Arc, Mutex};
+use std::rc::Rc;
 
 use anyhow::*;
 
@@ -43,6 +48,8 @@ use tracing::{error, info, warn};
 
 mod components;
 
+use crate::host_path;
+
 use self::components::{
     about::AboutDialog,
     albums::{
@@ -56,14 +63,14 @@ use self::components::{
     },
     library::{Library, LibraryInput, LibraryOutput},
     onboard::{Onboard, OnboardOutput},
-    preferences::{PreferencesDialog, PreferencesInput},
+    preferences::{PreferencesDialog, PreferencesInput, PreferencesOutput},
     viewer::view_nav::{ViewNav, ViewNavInput, ViewNavOutput},
 };
 
 mod background;
 
 use self::background::bootstrap::{
-    Bootstrap, BootstrapInput, BootstrapOutput, MediaType, TaskName,
+    Bootstrap, BootstrapInput, BootstrapOutput, MediaType, ThumbnailType, TaskName,
 };
 
 use self::components::progress_monitor::ProgressMonitor;
@@ -113,12 +120,16 @@ pub struct Settings {
     /// NOTE: doesn't include folder's album.
     pub album_sort: AlbumSort,
 
+    /// Enable processing of Android motion photos.
+    pub process_motion_photos: bool,
+
     /// Has the user completed the onboarding processes to select
     /// the picture library root directory?
     pub is_onboarding_complete: bool,
 
-    /// Base path of pictures directory.
-    pub pictures_base_dir: PathBuf,
+    /// Base path of pictures directory inside Flatpak sandbox.
+    /// Will be under `/run/users/<uid>/docs/<doc-id>/...`
+    pub library_base_dir: FlatpakPathBuf,
 }
 
 /// Active settings
@@ -236,6 +247,8 @@ pub(super) enum AppMsg {
     ScanPictureForFaces(PictureId),
     ScanPicturesForFaces,
 
+    ProcessMotionPhotos,
+
     // Stop all background tasks
     StopBackgroundTasks,
 
@@ -256,8 +269,8 @@ relm4::new_action_group!(pub(super) WindowActionGroup, "win");
 relm4::new_stateless_action!(PreferencesAction, WindowActionGroup, "preferences");
 relm4::new_stateless_action!(AboutAction, WindowActionGroup, "about");
 
-#[relm4::component(pub)]
-impl SimpleComponent for App {
+#[relm4::component(pub async)]
+impl SimpleAsyncComponent for App {
     type Init = ();
     type Input = AppMsg;
     type Output = ();
@@ -513,30 +526,37 @@ impl SimpleComponent for App {
         }
     }
 
-    fn init(
+    async fn init(
         _init: Self::Init,
         root: Self::Root,
-        sender: ComponentSender<Self>,
-    ) -> ComponentParts<Self> {
+        sender: AsyncComponentSender<Self>,
+    ) -> AsyncComponentParts<Self> {
         let data_dir = glib::user_data_dir().join(APP_ID);
         let _ = std::fs::create_dir_all(&data_dir);
 
         let cache_dir = glib::user_cache_dir().join(APP_ID);
         let _ = std::fs::create_dir_all(&cache_dir);
 
+        // WARN duplicate thumbnail path calculation in bootstrap.rs
+        let thumbnail_dir = glib::user_cache_dir()
+            .join(APP_ID) // Remove to use standard XDG thumbnail path
+            .join("thumbnails");
+
+        let thumbnailer = Rc::new(Thumbnailer::build(&thumbnail_dir));
+
         let db_path = data_dir.join("pictures.sqlite");
 
         let con = database::setup(&db_path).expect("Must be able to open database");
         let con = Arc::new(Mutex::new(con));
 
-        let people_repo = people::Repository::open(&data_dir, con.clone()).unwrap();
+        let people_repo = people::Repository::open(&cache_dir, &data_dir, con.clone()).unwrap();
 
         let state = SharedState::new(relm4::SharedState::new());
         let active_view = ActiveView::new(relm4::SharedState::new());
         let adaptive_layout = Arc::new(adaptive::LayoutState::new());
 
         let settings_state = SettingsState::new(relm4::SharedState::new());
-        match App::load_settings() {
+        match App::load_settings().await {
             std::result::Result::Ok(settings) => {
                 info!("Loaded settings: {:?}", settings);
                 *settings_state.write() = settings;
@@ -578,7 +598,7 @@ impl SimpleComponent for App {
         let onboard_view = adw::ToolbarView::new();
 
         let library = Library::builder()
-            .launch((state.clone(), active_view.clone(), adaptive_layout.clone()))
+            .launch((state.clone(), active_view.clone(), adaptive_layout.clone(), thumbnailer.clone()))
             .forward(sender.input_sender(), |msg| match msg {
                 LibraryOutput::View(id) => AppMsg::View(id, AlbumFilter::All),
             });
@@ -609,6 +629,7 @@ impl SimpleComponent for App {
                 active_view.clone(),
                 ViewName::Selfies,
                 AlbumFilter::Selfies,
+                thumbnailer.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
                 AlbumOutput::Selected(id, filter) => AppMsg::View(id, filter),
@@ -629,6 +650,7 @@ impl SimpleComponent for App {
                 active_view.clone(),
                 ViewName::Animated,
                 AlbumFilter::Motion,
+                thumbnailer.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
                 AlbumOutput::Selected(id, filter) => AppMsg::View(id, filter),
@@ -647,6 +669,7 @@ impl SimpleComponent for App {
                 active_view.clone(),
                 ViewName::Videos,
                 AlbumFilter::Videos,
+                thumbnailer.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
                 AlbumOutput::Selected(id, filter) => AppMsg::View(id, filter),
@@ -675,7 +698,7 @@ impl SimpleComponent for App {
         });
 
         let person_album = PersonAlbum::builder()
-            .launch((state.clone(), people_repo.clone(), active_view.clone()))
+            .launch((state.clone(), people_repo.clone(), active_view.clone(), thumbnailer.clone()))
             .forward(sender.input_sender(), |msg| match msg {
                 PersonAlbumOutput::Selected(id, filter) => AppMsg::View(id, filter),
                 PersonAlbumOutput::Deleted => AppMsg::PersonDeleted,
@@ -691,7 +714,7 @@ impl SimpleComponent for App {
         });
 
         let places_page = PlacesAlbum::builder()
-            .launch((state.clone(), active_view.clone()))
+            .launch((state.clone(), active_view.clone(), thumbnailer.clone()))
             .forward(sender.input_sender(), |msg| match msg {
                 PlacesAlbumOutput::View(visual_id) => {
                     AppMsg::View(visual_id.clone(), AlbumFilter::One(visual_id))
@@ -707,7 +730,7 @@ impl SimpleComponent for App {
         });
 
         let folders_album = FoldersAlbum::builder()
-            .launch((state.clone(), active_view.clone()))
+            .launch((state.clone(), active_view.clone(), thumbnailer.clone()))
             .forward(sender.input_sender(), |msg| match msg {
                 FoldersAlbumOutput::FolderSelected(path) => AppMsg::ViewFolder(path),
             });
@@ -723,6 +746,7 @@ impl SimpleComponent for App {
                 active_view.clone(),
                 ViewName::Folder,
                 AlbumFilter::None,
+                thumbnailer.clone(),
             ))
             .forward(sender.input_sender(), |msg| match msg {
                 AlbumOutput::Selected(id, filter) => AppMsg::View(id, filter),
@@ -739,7 +763,10 @@ impl SimpleComponent for App {
 
         let preferences_dialog = PreferencesDialog::builder()
             .launch((settings_state.clone(), root.clone()))
-            .detach();
+            .forward(sender.input_sender(), |msg| match msg {
+                PreferencesOutput::EnableFaceDetection => AppMsg::ScanPicturesForFaces,
+                PreferencesOutput::ProcessMotionPhotos=> AppMsg::ProcessMotionPhotos,
+            });
 
         let picture_navigation_view = adw::NavigationView::builder().build();
 
@@ -823,22 +850,20 @@ impl SimpleComponent for App {
 
         let settings = settings_state.read();
         let is_onboarding_complete =
-            settings.is_onboarding_complete && settings.pictures_base_dir.exists();
+            settings.is_onboarding_complete && settings.library_base_dir.exists();
         if is_onboarding_complete {
             model.picture_navigation_view.set_visible(true);
             model.onboard_view.set_visible(false);
-            model.bootstrap.emit(BootstrapInput::Configure(
-                settings.pictures_base_dir.clone(),
-            ));
+            sender.input(AppMsg::OnboardDone(settings.library_base_dir.sandbox_path.clone()));
         } else {
             model.picture_navigation_view.set_visible(false);
             model.onboard_view.set_visible(true);
         }
 
-        ComponentParts { model, widgets }
+        AsyncComponentParts { model, widgets }
     }
 
-    fn update(&mut self, message: Self::Input, sender: ComponentSender<Self>) {
+    async fn update(&mut self, message: Self::Input, sender: AsyncComponentSender<Self>) {
         match message {
             AppMsg::Activate(width) => {
                 if width >= 500 {
@@ -958,11 +983,8 @@ impl SimpleComponent for App {
                     TaskName::LoadLibrary => {
                         // do nothing
                     }
-                    TaskName::Scan(MediaType::Photo) => {
-                        self.banner.set_title(&fl!("banner-scan-photos"));
-                    }
-                    TaskName::Scan(MediaType::Video) => {
-                        self.banner.set_title(&fl!("banner-scan-videos"));
+                    TaskName::Scan => {
+                        self.banner.set_title(&fl!("banner-scan-library"));
                     }
                     TaskName::Enrich(MediaType::Photo) => {
                         self.banner.set_title(&fl!("banner-metadata-photos"));
@@ -973,12 +995,15 @@ impl SimpleComponent for App {
                     TaskName::MotionPhoto => {
                         self.banner.set_title(&fl!("banner-extract-motion-photos"));
                     }
-                    TaskName::Thumbnail(MediaType::Photo) => {
+                    TaskName::Thumbnail(ThumbnailType::Photo) => {
                         self.banner.set_title(&fl!("banner-thumbnails-photos"));
                     }
-                    TaskName::Thumbnail(MediaType::Video) => {
+                    TaskName::Thumbnail(ThumbnailType::Video) => {
                         self.banner.set_title(&fl!("banner-thumbnails-videos"));
                     }
+                    TaskName::Thumbnail(ThumbnailType::Face) => {
+                        self.banner.set_title(&fl!("banner-face-thumbnails"));
+                    }
                     TaskName::DetectFaces => {
                         self.banner.set_title(&fl!("banner-detect-faces-photos"));
                     }
@@ -994,6 +1019,12 @@ impl SimpleComponent for App {
                     TaskName::Transcode => {
                         self.banner.set_title(&fl!("banner-convert-videos"));
                     }
+                    TaskName::Tidy => {
+                        // Show nothing
+                    }
+                    TaskName::Migrate => {
+                        // Show nothing
+                    }
                 };
             }
             AppMsg::BootstrapCompleted => {
@@ -1014,6 +1045,10 @@ impl SimpleComponent for App {
                 info!("Scan pictures for faces");
                 self.bootstrap.emit(BootstrapInput::ScanPicturesForFaces);
             }
+            AppMsg::ProcessMotionPhotos => {
+                info!("Process motion photos");
+                self.bootstrap.emit(BootstrapInput::ProcessMotionPhotos);
+            }
             AppMsg::StopBackgroundTasks => {
                 info!("Stop all background tasks");
                 self.banner.set_button_label(None);
@@ -1040,13 +1075,14 @@ impl SimpleComponent for App {
                 // Notify of a change of layout.
                 *self.adaptive_layout.write() = adaptive::Layout::Wide;
             }
-            AppMsg::OnboardDone(pic_base_dir) => {
+            AppMsg::OnboardDone(library_base_dir) => {
                 let mut settings = self.settings_state.read().clone();
                 settings.is_onboarding_complete = true;
-                settings.pictures_base_dir = pic_base_dir.clone();
-                *self.settings_state.write() = settings;
+                settings.library_base_dir = host_path::host_path(&library_base_dir).await
+                    .unwrap_or(FlatpakPathBuf::build(&library_base_dir, &library_base_dir));
+                *self.settings_state.write() = settings.clone();
 
-                self.bootstrap.emit(BootstrapInput::Configure(pic_base_dir));
+                self.bootstrap.emit(BootstrapInput::Configure(settings.library_base_dir.clone()));
                 self.picture_navigation_view.set_visible(true);
                 self.onboard_view.set_visible(false);
             }
@@ -1059,11 +1095,20 @@ impl SimpleComponent for App {
 }
 
 impl App {
-    pub fn load_settings() -> Result<Settings> {
+    pub async fn load_settings() -> Result<Settings> {
         info!("Loading settings");
+
         let gio_settings = gio::Settings::new(APP_ID);
+
+        let pic_base_dir: PathBuf =  path_encoding::from_base64(
+                &gio_settings.string("pictures-base-dir-b64").into())?;
+
+        let library_base_dir = host_path::host_path(&pic_base_dir).await
+            .unwrap_or(FlatpakPathBuf::build(&pic_base_dir, &pic_base_dir));
+
         Ok(Settings {
             show_selfies: gio_settings.boolean("show-selfies"),
+            process_motion_photos: gio_settings.boolean("process-motion-photos"),
             face_detection_mode: FaceDetectionMode::from_str(
                 &gio_settings.string("face-detection-mode"),
             )
@@ -1071,9 +1116,7 @@ impl App {
             album_sort: AlbumSort::from_str(&gio_settings.string("album-sort"))
                 .unwrap_or(AlbumSort::Ascending),
             is_onboarding_complete: gio_settings.boolean("onboarding-complete"),
-            pictures_base_dir: path_encoding::from_base64(
-                &gio_settings.string("pictures-base-dir-b64").into(),
-            )?,
+            library_base_dir,
         })
     }
 
@@ -1081,12 +1124,13 @@ impl App {
         info!("Saving settings");
         let gio_settings = gio::Settings::new(APP_ID);
         gio_settings.set_boolean("show-selfies", settings.show_selfies)?;
+        gio_settings.set_boolean("process-motion-photos", settings.process_motion_photos)?;
         gio_settings.set_string("face-detection-mode", settings.face_detection_mode.as_ref())?;
         gio_settings.set_string("album-sort", settings.album_sort.as_ref())?;
         gio_settings.set_boolean("onboarding-complete", settings.is_onboarding_complete)?;
         gio_settings.set_string(
             "pictures-base-dir-b64",
-            &path_encoding::to_base64(settings.pictures_base_dir.as_ref()),
+            &path_encoding::to_base64(&settings.library_base_dir.sandbox_path),
         )?;
         Ok(())
     }
